// rsspecan_read_trace_example.cpp : implementation file
//

#include "stdafx.h"
#include "rsspecan_read_trace_example.h"
#include "rsspecan_read_trace_exampleDlg.h"
#include <fstream>
using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#define ARRAY_SIZE    5000         /* for rsspecan_ReadYTrace() varable: amplitude */
#define STRING_LENGTH 512
#define WINDOW        1


#define CHECKERR(fcal) if ((rsspecan_status = fcal) < 0)\
					   {\
							cleanup(VI_FALSE);\
							return;\
					   }


// CSPECANReadExampleDlg dialog




CSPECANReadExampleDlg::CSPECANReadExampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSPECANReadExampleDlg::IDD, pParent)
	, m_IDQuery(TRUE)
	, m_Reset(TRUE)
	, m_ResourceString(_T("GPIB::20::INSTR"))
	, m_Path(_T("c:\\trace.csv"))
	, m_CenterFrq(900e6)
	, m_FrqSpan(100e6)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CSPECANReadExampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Check(pDX, IDC_IDQUERY, m_IDQuery);
	DDX_Check(pDX, IDC_RESET, m_Reset);
	DDX_Text(pDX, IDC_RESOURCE_STRING, m_ResourceString);
	DDX_Text(pDX, IDC_PATH, m_Path);
	DDX_Text(pDX, IDC_CENTERFRQ, m_CenterFrq);
	DDX_Text(pDX, IDC_FRQSPAN, m_FrqSpan);
}

BEGIN_MESSAGE_MAP(CSPECANReadExampleDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_GENERATE, &CSPECANReadExampleDlg::OnBnClickedGenerate)
	ON_BN_CLICKED(IDC_BROWSE, &CSPECANReadExampleDlg::OnBnClickedBrowse)
END_MESSAGE_MAP()


// CSPECANReadExampleDlg message handlers

BOOL CSPECANReadExampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CSPECANReadExampleDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CSPECANReadExampleDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


void CSPECANReadExampleDlg::OnBnClickedGenerate()
{
		
	UpdateData ();
	HCURSOR cur = LoadCursor ( NULL, IDC_WAIT );
	SetCursor ( cur );
	ofstream	out_file;
    ViChar      instr_desc [STRING_LENGTH] = "";
    ViChar      instr_modl [STRING_LENGTH] = "";
	ViReal64    amplitude [ARRAY_SIZE];
	ViStatus	temp_status  = VI_SUCCESS; 
	ViInt32		actual_count = 0;

/************************************ TODO ****************************************/
    
	/* initialize driver */
	CHECKERR (rsspecan_init ((char *) ((LPCTSTR) m_ResourceString), m_IDQuery, m_Reset, &m_InstrSession));
                
	/* query instrument model */
    CHECKERR (rsspecan_GetAttributeViString (m_InstrSession, "", RS_ATTR_INSTRUMENT_MODEL, 
        STRING_LENGTH, instr_modl));
    
    /*
     * do not switch to spectrum analyzer mode if an FMU baseband analyzer is
     * connected. For this example the standard FFT-mode on the FMU is used
     */
    if(0 != strcmp("FMU", instr_modl))
        CHECKERR (rsspecan_SetSANMode (m_InstrSession));   
    
    /* configure specan device: center frequency and span */
	CHECKERR (rsspecan_ConfigureFrequencyCenterSpan (m_InstrSession, WINDOW, m_CenterFrq, m_FrqSpan));
    
    /* Set instrument to single sweep mode - continous mode is done
     * programmatically because this is the only way how to
     * synchronize measurement using *OPC
     */
    CHECKERR (rsspecan_ConfigureSignalStatisticSweep(m_InstrSession, VI_FALSE));


    /* Initiates a signal acquisition based on the present instrument configuration. 
     * It then waits for the acquisition to complete, and returns the trace as an 
     * array of amplitude values. */
    CHECKERR (rsspecan_ReadYTrace (m_InstrSession, 1, 1, 5000, ARRAY_SIZE, &actual_count, amplitude));

	out_file.open(m_Path);
	for(int i=0;i<actual_count;i++){
		out_file << amplitude[0] << "; ";
	}
	out_file.close();


	cur = LoadCursor (NULL, IDC_ARROW);
	SetCursor (cur);
	
	CHECKERR (rsspecan_close (m_InstrSession));
}

/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with error message.
/*           In case Device Clear is requested, it is necessary to set the
/*           argument deviceClear to VI_TRUE.
/*===========================================================================*/
void CSPECANReadExampleDlg::cleanup(ViBoolean deviceClear)
{
	if (rsspecan_status < 0)
	{
		ViChar      error_message[256];
		ViStatus    temp_status;
		CString		message;
		CString		error_string;
		
		/* Query the error message from the instrument driver. This
		 * function gets the error message for VISA and driver failures
		 */
		temp_status = rsspecan_error_message(m_InstrSession, rsspecan_status, error_message);
		
		if (temp_status == VI_WARN_UNKNOWN_STATUS)
		{
			/* Query the error message from the instrument. This
			 * function gets the error message directly from instrument, like
			 * execution error, out of range error etc.
			 */
			rsspecan_error_query(m_InstrSession, &temp_status, error_message);
		}
		error_string = error_message;
		message.Format (_T ("Error 0x%8X occured:\n\n%s"), rsspecan_status, error_string);
		AfxMessageBox (message, MB_OK | MB_ICONSTOP); 
	}
	
	/* Device Clear is a low-level command and should be used in case the
	 * instrument is waiting for Operation Complete to cancel the wait.
	 * It is useful for instance in case of incorrect external trigger
	 * when the instrument does not respond to any other command because of
	 * waiting for trigger.
	 */
	if (deviceClear)
	{
		viClear (m_InstrSession);
		viPrintf (m_InstrSession, "*CLS\n");
	}
	
	rsspecan_close (m_InstrSession);

	HCURSOR cur = LoadCursor ( NULL, IDC_ARROW );
	SetCursor (cur);
}
void CSPECANReadExampleDlg::OnBnClickedBrowse()
{
	CFileDialog fileDlg( FALSE);

	if ( fileDlg.DoModal() == IDOK)
	{
		m_Path = fileDlg.GetPathName();
		UpdateData(FALSE);
	}
}



